# Generated from Zish.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .ZishParser import ZishParser
else:
    from ZishParser import ZishParser

# This class defines a complete listener for a parse tree produced by ZishParser.
class ZishListener(ParseTreeListener):

    # Enter a parse tree produced by ZishParser#start.
    def enterStart(self, ctx:ZishParser.StartContext):
        pass

    # Exit a parse tree produced by ZishParser#start.
    def exitStart(self, ctx:ZishParser.StartContext):
        pass


    # Enter a parse tree produced by ZishParser#element.
    def enterElement(self, ctx:ZishParser.ElementContext):
        pass

    # Exit a parse tree produced by ZishParser#element.
    def exitElement(self, ctx:ZishParser.ElementContext):
        pass


    # Enter a parse tree produced by ZishParser#list_type.
    def enterList_type(self, ctx:ZishParser.List_typeContext):
        pass

    # Exit a parse tree produced by ZishParser#list_type.
    def exitList_type(self, ctx:ZishParser.List_typeContext):
        pass


    # Enter a parse tree produced by ZishParser#set_type.
    def enterSet_type(self, ctx:ZishParser.Set_typeContext):
        pass

    # Exit a parse tree produced by ZishParser#set_type.
    def exitSet_type(self, ctx:ZishParser.Set_typeContext):
        pass


    # Enter a parse tree produced by ZishParser#map_type.
    def enterMap_type(self, ctx:ZishParser.Map_typeContext):
        pass

    # Exit a parse tree produced by ZishParser#map_type.
    def exitMap_type(self, ctx:ZishParser.Map_typeContext):
        pass


    # Enter a parse tree produced by ZishParser#pair.
    def enterPair(self, ctx:ZishParser.PairContext):
        pass

    # Exit a parse tree produced by ZishParser#pair.
    def exitPair(self, ctx:ZishParser.PairContext):
        pass


    # Enter a parse tree produced by ZishParser#key.
    def enterKey(self, ctx:ZishParser.KeyContext):
        pass

    # Exit a parse tree produced by ZishParser#key.
    def exitKey(self, ctx:ZishParser.KeyContext):
        pass


    # Enter a parse tree produced by ZishParser#ws.
    def enterWs(self, ctx:ZishParser.WsContext):
        pass

    # Exit a parse tree produced by ZishParser#ws.
    def exitWs(self, ctx:ZishParser.WsContext):
        pass


